[BITS 16]

   jmp real_start

%include "zsfs.inc"



real_start:
   mov ax, 0x07c0 ;set-up segments
   mov ds, ax
   mov es, ax
   mov fs, ax
   mov gs, ax

   cli
   xor ax, ax ;set-up stack
   mov ss, ax
   mov sp, 0xFFFF
   sti

   mov byte[BOOTD], dl ;save the boot device number.

   mov ax, word[FileEntryStart]
   call lba2hts

   mov ax, word[FileEntrySize]
   mov ah, 2
   mov bx, 0x0200
   pusha
   int 0x13
   popa
   jnc find_file
   mov si, error1
   jmp error

find_file:
   push ds
   pop es
   mov di, 0x0200
   mov cx, 256
find_loop:
   xchg cx, dx
   mov si, file2load
   mov cx, 11
   push di
   rep cmpsb
   pop di
   je file_found
   add di, 128
   xchg dx, cx
   loop find_loop
   mov si, error2
   jmp error

file_found:
   mov ax, word[es:di+64] ;First64=FileEntry, Second64=I-Tree
   mov word[CLUSTER], ax

   mov ax, word[FileEntryStart]
   add ax, word[FileEntrySize]
   call lba2hts

   mov ax, word[ClusterChainSize]

   push ds
   pop es

   mov ah, 2
   mov bx, 0x0200
   pusha
   int 0x13
   popa
   jnc FindCluster
   mov si, error1
   jmp error

FindCluster:
   mov ax, word[CLUSTER] ;the pointer in cluster chain.
   mov bx, 128 ;each entry is 128 in size
   dec ax 
   mul bx

   add di, ax
   mov ax, word[es:di] ; first cluster
   add ax, 31
   call lba2hts

   xor bx, bx
   mov es, bx 
   mov bx, 0x0500 ;0x0000:0x0500

   mov ah, 2
   mov al, 23 ;24 sectors
   pusha
   int 0x13
   popa
   jnc execute_it
   mov si, error1
   jmp error
execute_it:
   mov dl, byte[BOOTD]
   jmp 0x0000:0x0500
   
   
err3:
   mov si, error3
error:
   mov ah, 0x0e
.loop:
   lodsb
   or al, al
   jz .done
   int 0x10
   jmp .loop
.done:
   cli
   hlt
   jmp $




lba2hts:			; Calculate head, track and sector settings for int 13h
			; IN: logical sector in AX, OUT: correct registers for int 13h
	push bx
	push ax

	mov bx, ax			; Save logical sector

	mov dx, 0			; First the sector
	div word [SectorsPerTrack]
	add dl, 01h			; Physical sectors start at 1
	mov cl, dl			; Sectors belong in CL for int 13h
	mov ax, bx

	mov dx, 0			; Now calculate the head
	div word [SectorsPerTrack]
	mov dx, 0
	div word [HeadsPerCylinder]
	mov dh, dl			; Head/side
	mov ch, al			; Track

	pop ax
	pop bx

	mov dl, byte [BOOTD]		; Set correct device

	ret

BOOTD:   db 0
CLUSTER: dw 0
FileSize:  db 0
file2load: db "KERNLDR SYS"
error1:  db "DiskErr",0
error2:  db "File not found.",0
error3:  db "Kernel fault.", 0 

times 510-($-$$) db 0
dw 0xAA55
filebuff: