;Common definitions in file systems
SectorsPerCluster:      dw 1
SectorsPerTrack:        dd 18
HeadsPerCylinder:       dd 2
BytesPerSector:         dw 512
;Custom definitions
BytesPerEntry:          dw 2
FileEntryStart:         dw 2
entries_indisk:         dq 256
FileEntrySize:          dq 64 ;Size in LBA
ClusterChainSize:       dq 64 ;Size in LBA
TotalSectors64:         dq 2880
;---------------------------------------
BOOTD:   db 0
CLUSTER: dw 0
;---------------------------------------

LoadEntries:
    pusha
    push es

    mov ax, word[FileEntryStart]
    call lba2hts

    xor bx, bx
    mov es, bx
    mov bx, 0x7E00

    mov ax, word[FileEntrySize]
    mov ah, 2
    mov dl, byte[BOOTD]
    int 0x13

    pop es
    popa
    ret

;----------------------------------------------------------------------------------
 
LoadFile:
	push bx
	push bp ;save for later use

	mov bx, si ;save for later use
	mov cx, 256 ;256 entries
	xor ax, ax
	mov es, ax
	mov di, 0x7E00
.findLoop:
	xchg cx, dx
	mov si, bx
	mov cx, 11 ;file name in 8.3 format is like 11 bytes :b
	push di
	rep cmpsb
	pop di
	je .foundFile
	add di, 128 ;next entry
	xchg dx, cx
	loop .findLoop
	jmp .error
.foundFile:

	mov ax, word[es:di+64] ;Fist64Bytes=FileEntry Second64Bytes=I-Tree
	mov word[CLUSTER], ax
	;lets load the C-Chain from the file in memory.
	mov ax, word[FileEntryStart]
	add ax, word[FileEntrySize] ;point to I-Trees
	call lba2hts

	xor bx, bx
	mov es, bx
	mov bx, 0x7E00

	mov ax, word[ClusterChainSize]
	mov ah, 0x02
	mov dl, byte[BOOTD]
	pusha
	int 0x13
	popa
.PrepareForChain:
   mov ax, word[CLUSTER] ;the pointer in cluster chain.
   mov bx, 128 ;each entry is 128 in size
   dec ax 
   mul bx

   mov di, 0x7E00
   add di, ax ;The actual cluster chain.
   ;now we are prepared huh?

   pop bp
   pop bx
   mov es, bx ;bx:bp
   mov bx, bp

   mov ecx, 0 ;size in 
   push ecx
.loadLoop:
	mov ax, word[di]
	cmp ax, 0
	je .checkerr
	add ax, 31
	call lba2hts

	mov ah, 2
	mov al, 1
	mov dl, byte[BOOTD]
	pusha
	int 0x13
	popa
	add bx, 512
	add di, word[BytesPerEntry]
	pop ecx
	inc ecx
	push ecx
	cmp ecx, 64
	jne .loadLoop



.checkerr:
	pop ecx
	cmp ecx, 0
	jne .success
	push bx
	push bp
.error:
	pop bp
	pop bx
	mov ax, -1
.success:
	ret








;----------------------------------------------------------------------------------





lba2hts:			; Calculate head, track and sector settings for int 13h
			; IN: logical sector in AX, OUT: correct registers for int 13h
	push bx
	push ax

	mov bx, ax			; Save logical sector

	mov dx, 0			; First the sector
	div word [SectorsPerTrack]
	add dl, 01h			; Physical sectors start at 1
	mov cl, dl			; Sectors belong in CL for int 13h
	mov ax, bx

	mov dx, 0			; Now calculate the head
	div word [SectorsPerTrack]
	mov dx, 0
	div word [HeadsPerCylinder]
	mov dh, dl			; Head/side
	mov ch, al			; Track

	pop ax
	pop bx

	mov dl, byte [BOOTD]		; Set correct device

	ret